# Copyright (c) 2009 Entertainer Developers - See COPYING - GPLv2
'''FeedLibrary - Library for RSS-feeds'''

from pysqlite2 import dbapi2 as sqlite

from entertainerlib.configuration import Configuration

class FeedLibrary(object):
    '''This library can be used to handle RSS feeds in Entertainer.'''

    def __init__(self, backend_connection):
        self.config = Configuration()

        self.backend_connection = backend_connection
        self.empty = False
        if self.number_of_feeds() == 0:
            self.empty = True

    def is_empty(self):
        '''Boolean used to indicated if the feed library is empty.'''
        return self.empty

    def get_feeds(self, order='TITLE'):
        '''Return a list of Feed objects from the library based on the order of
        TITLE or UNREAD.'''
        if self.is_empty():
            return []
        else:
            feeds = []
            connection = sqlite.connect(self.config.FEED_DB)
            cursor = connection.cursor()
            cursor.execute("SELECT url FROM feed ORDER BY title")
            for row in cursor:
                feeds.append(Feed(row[0]))
            connection.close()
            if order == "UNREAD":
                feeds.sort(self.compare_feeds_by_unread)
            return feeds

    def compare_feeds_by_unread(self, feed_x, feed_y):
        '''Compare which feed has more unread and return a value that follows
        Python's cmp convention of +, 0, or -.'''
        return cmp(
            feed_y.get_number_of_unread(),
            feed_x.get_number_of_unread())

    def get_latest_entries(self, max_return=10):
        '''Return the latest entries as a list of tuples with each tuple
        containing a Feed object and an Entry object.'''
        if self.is_empty():
            return []
        else:
            latest = []
            connection = sqlite.connect(self.config.FEED_DB)
            cursor = connection.cursor()
            cursor.execute("""SELECT DISTINCT feed.url, entry.id
                              FROM feed, entry
                              WHERE entry.feed_url = feed.url
                              ORDER BY entry.date DESC, entry.time  DESC
                              LIMIT 0,:max""",
                              { "max" : max_return } )
            for row in cursor:
                feed = Feed(row[0])
                entry = feed.get_entry(row[1])
                latest.append((feed, entry))
            connection.close()
            return latest

    def number_of_feeds(self):
        '''Return the number of feeds in the library.'''
        if self.is_empty():
            return 0
        else:
            connection = sqlite.connect(self.config.FEED_DB)
            cursor = connection.cursor()
            cursor.execute("SELECT COUNT(url) FROM feed")
            result = cursor.fetchall()
            connection.close()
            return result[0][0]

    def number_of_unread_entries(self):
        '''Return the number of unread entries.'''
        if self.is_empty():
            return 0
        else:
            connection = sqlite.connect(self.config.FEED_DB)
            cursor = connection.cursor()
            cursor.execute(
                """SELECT COUNT(feed_url) FROM entry WHERE isread='NO'""")
            result = cursor.fetchall()
            connection.close()
            return result[0][0]

    def request_feed_update(self):
        '''Request backend to fetch and update all the feeds.'''
        self.backend_connection.request_feed_update()

    def mark_all_as_read(self):
        '''Mark all feed entries as read.'''
        for feed in self.get_feeds():
            for entry in feed.entries:
                entry.read = True


class Feed(object):
    '''Representation of a RSS feed.'''

    def __init__(self, url):
        self.config = Configuration()

        self.url = url
        self.entries = []
        connection = sqlite.connect(self.config.FEED_DB)
        cursor = connection.cursor()
        cursor.execute("""SELECT title, subtitle, description, time, date
                          FROM feed
                          WHERE url=:url""",
                          { "url" : self.url })
        result = cursor.fetchall()
        self.title = result[0][0]
        self.subtitle = result[0][1]
        self.description = result[0][2]
        self.time = result[0][3]
        self.date = result[0][4]
        cursor.execute("""SELECT title, description, id, time, date, isread
                          FROM entry
                          WHERE entry.feed_url=:url
                          ORDER BY date DESC, time DESC""",
                          { "url" : self.url })
        for row in cursor:
            if row[5] == "YES":
                isread = True
            else:
                isread = False
            self.entries.append(
                Entry(row[0], row[1], row[2], row[3], row[4], isread))
        connection.close()

    def get_entry(self, identifier):
        '''Return an entry with the provided identifier.'''
        for entry in self.entries:
            if entry.identifier == identifier:
                return entry

    def get_number_of_unread(self):
        '''Return the number of unread entries.'''
        connection = sqlite.connect(self.config.FEED_DB)
        cursor = connection.cursor()
        cursor.execute("""SELECT COUNT(feed_url)
                          FROM   entry
                          WHERE  isread='NO'
                          AND    feed_url=:url""",
                          { "url" : self.url })
        result = cursor.fetchall()
        connection.close()
        return int(result[0][0])


class Entry(object):
    '''Representation of an individual RSS feed entry.'''

    def __init__(self, title, description, identifier, time, date, read=False):
        self.config = Configuration()
        self.title = title
        self.description = description
        self.identifier = identifier
        self.time = time
        self.date = date
        self._read = read

    def _get_read(self):
        '''Get the read property.'''
        return self._read

    def _set_read(self, read):
        '''Set the read property and update the database.'''
        self._read = read

        if read:
            readval = "YES"
        else:
            readval = "NO"
        connection = sqlite.connect(self.config.FEED_DB)
        cursor = connection.cursor()
        cursor.execute(
            """UPDATE entry SET isread=:val WHERE id=:id""",
            {"val" : readval, "id" : self.identifier})
        connection.commit()
        connection.close()

    read = property(_get_read, _set_read)

