# Copyright (c) 2009 Entertainer Developers - See COPYING - GPLv2
'''Feed - Screen that contains a list of entries in one feed'''

import pango

from entertainerlib.gui.screens.screen import Screen
from entertainerlib.gui.widgets.label import Label
from entertainerlib.gui.widgets.list_indicator import ListIndicator
from entertainerlib.gui.widgets.texture import Texture
from entertainerlib.gui.widgets.text_menu import TextMenu
from entertainerlib.backend.components.feeds.feed_utils import FeedEntryParser

class Feed(Screen):
    '''Screen is displayed when headlines are accessed for a specific feed.'''

    def __init__(self, feed, move_to_new_screen_callback):
        Screen.__init__(self, 'Feed', move_to_new_screen_callback)

        self.theme = self.config.theme
        self.feed = feed
        self.menu = None

        self.menu = self._create_entry_menu()
        self.add(self.menu)

        # Screen Title (Displayed at the bottom left corner)
        screen_title = Label(0.13, "screentitle", 0, 0.87, _("Feed"))
        self.add(screen_title)

        # RSS Icon
        icon = Texture(self.theme.getImage("rss_icon"), 0.05, 0.07)
        self.add(icon)

        # RSS Feed Title
        title_text = FeedEntryParser().convert(self.feed.title)
        title = Label(0.065, "title", 0.13, 0.07, title_text)
        title.set_use_markup(True)
        title.width = 0.6
        title.set_ellipsize(pango.ELLIPSIZE_END)
        self.add(title)

        # RSS Feed description
        desc_text = FeedEntryParser().convert(self.feed.subtitle)
        desc = Label(0.03646, "subtitle", 0.13, 0.14, desc_text)
        desc.set_use_markup(True)
        desc.width = 0.6
        desc.set_ellipsize(pango.ELLIPSIZE_END)
        self.add(desc)

        # Last update
        update = Label(0.0313, "title", 0.78, 0.1, _("Updated"))
        self.add(update)
        date_text = self.feed.time[:5]  + " - " +  self.feed.date
        date = Label(0.0313, "subtitle", 0.78, 0.14, date_text)
        self.add(date)

        #List indicator
        self.li = ListIndicator(0.57, 0.9, 0.2, 0.045, ListIndicator.VERTICAL)
        self.li.set_maximum(len(self.feed.entries))
        self.add(self.li)

        self._display_selected_feed()

        self.menu.connect('selected', self._on_menu_selected)
        self.menu.connect('moved', self._display_selected_feed)

    def _create_entry_menu(self):
        """Create Entries-menu. This menu contains list of entries."""
        menu = TextMenu(0.2438, 0.2604, 0.5124, 0.0781)

        parser = FeedEntryParser()
        feeds = self.feed.entries
        feeds_list = [[parser.strip_tags(feed.title), None, feed] \
            for feed in feeds]
        menu.async_add(feeds_list)

        menu.active = True

        return menu

    def _handle_up(self):
        '''Handle UserEvent.NAVIGATE_UP.'''
        self.menu.up()

    def _handle_down(self):
        '''Handle UserEvent.NAVIGATE_DOWN.'''
        self.menu.down()

    def _handle_select(self, event=None):
        '''Handle UserEvent.NAVIGATE_SELECT.'''
        entry = self.menu.selected_userdata
        kwargs = { 'feed' : self.feed, 'entry' : entry }
        self.callback("entry", kwargs)

    def _on_menu_selected(self, event=None):
        '''Handle a *select command* if an item was selected.'''
        self._handle_select()

    def _display_selected_feed(self, event=None):
        '''Update of the list indicator'''
        self.li.set_current(self.menu.selected_index + 1)

