# Copyright (c) 2009 Entertainer Developers - See COPYING - GPLv2
'''Rss - Screen displays RSS feeds'''

import pango

from entertainerlib.gui.screens.screen import Screen
from entertainerlib.gui.widgets.label import Label
from entertainerlib.gui.widgets.texture import Texture
from entertainerlib.gui.widgets.text_menu import TextMenu
from entertainerlib.gui.widgets.list_indicator import ListIndicator
from entertainerlib.backend.components.feeds.feed_utils import FeedEntryParser

class Rss(Screen):
    '''Screen displays RSS-feed titles and number of entries.'''

    def __init__(self, feed_library, move_to_new_screen_callback):
        Screen.__init__(self, 'Rss', move_to_new_screen_callback)

        self.feed_order = 'UNREAD' # Feeds ordered by number of unread entries
        self.theme = self.config.theme
        self.library = feed_library
        self.parser = FeedEntryParser()

        # Screen Title (Displayed at the bottom left corner)
        screen_title = Label(0.13, "screentitle", 0, 0.87, _("Headlines"))
        self.add(screen_title)

        # RSS Icon
        icon = Texture(self.theme.getImage("rss_icon"), 0.05, 0.07)
        self.add(icon)

        # RSS Title
        title = Label(0.0833, "title", 0.13, 0.07, _("RSS Feeds"))
        self.add(title)

        #We set the menus to None here so we can check if they exist later on
        self.menu = None
        self.context_menu = None

        # Display information box if there are not feeds available
        if self.library.is_empty() or len(self.library.get_feeds()) == 0:
            self._create_no_feeds_information()
        else:
            # Menus
            self._create_context_menu()
            self.menu = self._create_feed_menu()
            self.add(self.menu)
            self.li = None
            self._create_feed_menu_indicator()
            self.menu.active = True
            self.context_menu.active = False

            # Feeds Title
            feeds_title = Label(0.04167, "title", 0.3953, 0.2, _("Feeds"))
            self.add(feeds_title)

            # Unread Title
            unread_title = Label(0.04167, "title", 0.8053, 0.2, _("Unread"))
            self.add(unread_title)

            # Feed description text
            desc_text = self.parser.strip_tags(
                self.menu.selected_userdata.description)

            self.description = Label(0.03646, "text", 0.3953, 0.9, desc_text)
            self.description.set_ellipsize(pango.ELLIPSIZE_END)
            self.description.set_use_markup(True)
            self.description.set_size(0.5088, 0.0651)
            self.add(self.description)

            self.menu.connect('selected', self._handle_select)
            self.menu.connect('moved', self._display_selected_feed)
            self.menu.connect('activated', self._menu_activation)
            self.context_menu.connect('selected', self._handle_select)
            self.context_menu.connect('activated',
                self._context_menu_activation)

    def _create_feed_menu(self):
        """
        Create Feed-menu. This menu contains list of Feeds. It also displays
        number of unread entries per feed.
        """
        menu = TextMenu(0.3807, 0.2604, 0.5124, 0.0781)

        feeds = self.library.get_feeds(self.feed_order)
        feeds_list = [[self.parser.strip_tags(feed.title), \
            str(feed.get_number_of_unread()), feed] \
            for feed in feeds]
        menu.async_add(feeds_list)

        return menu

    def _create_feed_menu_indicator(self):
        '''Create a ListIndicator for feed menu.'''
        self.li = ListIndicator(0.7, 0.9, 0.2, 0.045, ListIndicator.VERTICAL)
        self.li.set_maximum(len(self.library.get_feeds(self.feed_order)))
        self.add(self.li)

    def _create_context_menu(self):
        """
        Create RSS-feed context menu and add it to the screen.
        This menu contains buttons for update and sorting.
        """
        self.context_menu = TextMenu(0.0732, 0.2604, 0.2196, 0.0781)

        self.context_menu.add_item(_("Update feeds"))
        self.context_menu.add_item(_("Mark all as read"))
        self.context_menu.add_item(_("Sort by name"))
        self.context_menu.add_item(_("Sort by unread"))

        self.context_menu.active = False
        self.add(self.context_menu)

    def _create_no_feeds_information(self):
        """
        Create textures and labels for information screen. This is displayed
        instead of feeds list if there are no feeds available. This screen helps
        users to add new feeds to the system.
        """
        # Create warning icon
        warning_icon = Texture(self.theme.getImage("warning_icon"), 0.28, 0.27)
        self.add(warning_icon)

        # Create warning title
        info_title = Label(0.0625, "title", 0.3367, 0.2709,
            _("No headlines available!"))
        self.add(info_title)

        # Create warning help text
        message = _(
            "To use headlines you have to add some RSS feeds with the Content "
            "management tool. To do this, start the Content management tool, "
            "open the 'RSS Feeds' tab and click the 'Add' button. Now you can "
            "write an RSS feed URL in the text entry. You can get RSS feed "
            "URLs from webpages that support RSS.")
        info = Label(0.0417, "menuitem_inactive", 0.2804, 0.45, message)
        info.set_size(0.5, 0.5208)
        self.add(info)

    def update(self, event=None):
        """
        Update screen widgets. This is called always when screen is poped from
        the screen history.
        """
        feeds = self.library.get_feeds(self.feed_order)
        for idx, item in enumerate(self.menu.items):
            feed = feeds[idx]
            item.userdata = feed
            item.update(self.parser.strip_tags(feed.title),
                str(feed.get_number_of_unread()))

    def _handle_up(self):
        '''Handle UserEvent.NAVIGATE_UP.'''
        if self.menu.active:
            self.menu.up()
        else:
            self.context_menu.up()

    def _handle_down(self):
        '''Handle UserEvent.NAVIGATE_DOWN.'''
        if self.menu.active:
            self.menu.down()
        else:
            self.context_menu.down()

    def _handle_left(self):
        '''Handle UserEvent.NAVIGATE_LEFT.'''
        if self.menu.active:
            self._context_menu_activation()

    def _handle_right(self):
        '''Handle UserEvent.NAVIGATE_RIGHT.'''
        if not self.menu.active:
            self._menu_activation()

    def _handle_select(self, event=None):
        '''Handle UserEvent.NAVIGATE_SELECT.'''
        if not self.menu.active:
            index = self.context_menu.selected_index
            if index == 0:
                # Send message to bus and update screen
                self.library.request_feed_update()
                self.update()
            elif index == 1:
                # Tell feedlibrary to mark all as read and then update screen
                self.library.mark_all_as_read()
                self.update()
            elif index == 2:
                # Sort feeds by title ASC and update screen
                self.feed_order = 'TITLE'
                self.update()
            elif index == 3:
                # Sort feeds by UNREAD DESC and update screen
                self.feed_order = 'UNREAD'
                self.update()
        else:
            # Feed selected from the menu. Change screen.
            feed = self.menu.selected_userdata
            kwargs = { 'feed' : feed }
            self.callback("feed", kwargs)

    def handle_user_event(self, event=None):
        '''Handle screen specific user events unless the library is empty.'''
        if self.library.is_empty() or len(self.library.get_feeds()) == 0:
            return
        else:
            Screen.handle_user_event(self, event)

    def _display_selected_feed(self, actor=None):
        '''Update of the list indicator and the description label'''
        self.li.set_current(self.menu.selected_index + 1)
        feed = self.menu.selected_userdata
        self.description.set_text(self.parser.strip_tags(feed.description))

    def _menu_activation(self, actor=None):
        '''Handle the menu activation'''
        self.menu.active = True
        self.context_menu.active = False
        self.li.show_position()
        self.description.show()

    def _context_menu_activation(self, actor=None):
        '''Handle the context menu activation'''
        self.menu.active = False
        self.context_menu.active = True
        self.li.hide_position()
        self.description.hide()

